@@ -15,10 +15,15 @@ var util = require('util')
  * HyBi Receiver implementation
  */
 
-function Receiver (extensions) {
+function Receiver (extensions,maxPayload) {
   if (this instanceof Receiver === false) {
     throw new TypeError("Classes can't be function-called");
   }
+  if(typeof extensions==='number'){
+    maxPayload=extensions;
+    extensions={};
+  }
+
 
   // memory pool for fragmented messages
   var fragmentedPoolPrevUsed = -1;
@@ -39,8 +44,9 @@ function Receiver (extensions) {
       Math.ceil((unfragmentedPoolPrevUsed + db.used) / 2) :
       db.used;
   });
-
   this.extensions = extensions || {};
+  this.maxPayload = maxPayload || 0;
+  this.currentPayloadLength = 0;
   this.state = {
     activeFragmentedOperation: null,
     lastFragment: false,
@@ -54,6 +60,7 @@ function Receiver (extensions) {
   this.expectBuffer = null;
   this.expectHandler = null;
   this.currentMessage = [];
+  this.currentMessageLength = 0;
   this.messageHandlers = [];
   this.expectHeader(2, this.processPacket);
   this.dead = false;
@@ -76,6 +83,7 @@ module.exports = Receiver;
  */
 
 Receiver.prototype.add = function(data) {
+  if (this.dead) return;
   var dataLength = data.length;
   if (dataLength == 0) return;
   if (this.expectBuffer == null) {
@@ -244,6 +252,7 @@ Receiver.prototype.processPacket = function (data) {
  */
 
 Receiver.prototype.endPacket = function() {
+  if (this.dead) return;
   if (!this.state.fragmentedOperation) this.unfragmentedBufferPool.reset(true);
   else if (this.state.lastFragment) this.fragmentedBufferPool.reset(true);
   this.expectOffset = 0;
@@ -253,6 +262,7 @@ Receiver.prototype.endPacket = function() {
     // end current fragmented operation
     this.state.activeFragmentedOperation = null;
   }
+  this.currentPayloadLength = 0;
   this.state.lastFragment = false;
   this.state.opcode = this.state.activeFragmentedOperation != null ? this.state.activeFragmentedOperation : 0;
   this.state.masked = false;
@@ -281,7 +291,9 @@ Receiver.prototype.reset = function() {
   this.expectHandler = null;
   this.overflow = [];
   this.currentMessage = [];
+  this.currentMessageLength = 0;
   this.messageHandlers = [];
+  this.currentPayloadLength = 0;
 };
 
 /**
@@ -296,20 +308,6 @@ Receiver.prototype.unmask = function (mask, buf, binary) {
   return buf != null ? buf.toString('utf8') : '';
 };
 
-/**
- * Concatenates a list of buffers.
- *
- * @api private
- */
-
-Receiver.prototype.concatBuffers = function(buffers) {
-  var length = 0;
-  for (var i = 0, l = buffers.length; i < l; ++i) length += buffers[i].length;
-  var mergedBuffer = new Buffer(length);
-  bufferUtil.merge(mergedBuffer, buffers);
-  return mergedBuffer;
-};
-
 /**
  * Handles an error
  *
@@ -317,8 +315,17 @@ Receiver.prototype.concatBuffers = function(buffers) {
  */
 
 Receiver.prototype.error = function (reason, protocolErrorCode) {
+  if (this.dead) return;
   this.reset();
-  this.onerror(reason, protocolErrorCode);
+  if(typeof reason == 'string'){
+    this.onerror(new Error(reason), protocolErrorCode);
+  }
+  else if(reason.constructor == Error){
+    this.onerror(reason, protocolErrorCode);
+  }
+  else{
+    this.onerror(new Error("An error occured"),protocolErrorCode);
+  }
   return this;
 };
 
@@ -365,6 +372,27 @@ Receiver.prototype.applyExtensions = function(messageBuffer, fin, compressed, ca
   }
 };
 
+/**
+* Checks payload size, disconnects socket when it exceeds maxPayload
+*
+* @api private
+*/
+Receiver.prototype.maxPayloadExceeded = function(length) {
+  if (this.maxPayload=== undefined || this.maxPayload === null || this.maxPayload < 1) {
+    return false;
+  }
+  var fullLength = this.currentPayloadLength + length;
+  if (fullLength < this.maxPayload) {
+    this.currentPayloadLength = fullLength;
+    return false;
+  }
+  this.error('payload cannot exceed ' + this.maxPayload + ' bytes', 1009);
+  this.messageBuffer=[];
+  this.cleanup();
+
+  return true;
+};
+
 /**
  * Buffer utilities
  */
@@ -425,11 +453,20 @@ var opcodes = {
       // decode length
       var firstLength = data[1] & 0x7f;
       if (firstLength < 126) {
+        if (self.maxPayloadExceeded(firstLength)){
+          self.error('Maximumpayload exceeded in compressed text message. Aborting...', 1009);
+          return;
+        }
         opcodes['1'].getData.call(self, firstLength);
       }
       else if (firstLength == 126) {
         self.expectHeader(2, function(data) {
-          opcodes['1'].getData.call(self, readUInt16BE.call(data, 0));
+          var length = readUInt16BE.call(data, 0);
+          if (self.maxPayloadExceeded(length)){
+            self.error('Maximumpayload exceeded in compressed text message. Aborting...', 1009);
+            return;
+          }
+          opcodes['1'].getData.call(self, length);
         });
       }
       else if (firstLength == 127) {
@@ -438,6 +475,11 @@ var opcodes = {
             self.error('packets with length spanning more than 32 bit is currently not supported', 1008);
             return;
           }
+          var length = readUInt32BE.call(data, 4);
+          if (self.maxPayloadExceeded(length)){
+            self.error('Maximumpayload exceeded in compressed text message. Aborting...', 1009);
+            return;
+          }
           opcodes['1'].getData.call(self, readUInt32BE.call(data, 4));
         });
       }
@@ -464,12 +506,29 @@ var opcodes = {
       var state = clone(this.state);
       this.messageHandlers.push(function(callback) {
         self.applyExtensions(packet, state.lastFragment, state.compressed, function(err, buffer) {
-          if (err) return self.error(err.message, 1007);
-          if (buffer != null) self.currentMessage.push(buffer);
-
+          if (err) {
+            if(err.type===1009){
+                return self.error('Maximumpayload exceeded in compressed text message. Aborting...', 1009);
+            }
+            return self.error(err.message, 1007);
+          }
+          if (buffer != null) {
+            if( self.maxPayload==0 || (self.maxPayload > 0 && (self.currentMessageLength + buffer.length) < self.maxPayload) ){
+              self.currentMessage.push(buffer);
+            }
+            else{
+                self.currentMessage=null;
+                self.currentMessage = [];
+                self.currentMessageLength = 0;
+                self.error(new Error('Maximum payload exceeded. maxPayload: '+self.maxPayload), 1009);
+                return;
+            }
+            self.currentMessageLength += buffer.length;
+          }
           if (state.lastFragment) {
-            var messageBuffer = self.concatBuffers(self.currentMessage);
+            var messageBuffer = Buffer.concat(self.currentMessage);
             self.currentMessage = [];
+            self.currentMessageLength = 0;
             if (!Validation.isValidUTF8(messageBuffer)) {
               self.error('invalid utf8 sequence', 1007);
               return;
@@ -490,11 +549,20 @@ var opcodes = {
       // decode length
       var firstLength = data[1] & 0x7f;
       if (firstLength < 126) {
+          if (self.maxPayloadExceeded(firstLength)){
+            self.error('Max payload exceeded in compressed text message. Aborting...', 1009);
+            return;
+          }
         opcodes['2'].getData.call(self, firstLength);
       }
       else if (firstLength == 126) {
         self.expectHeader(2, function(data) {
-          opcodes['2'].getData.call(self, readUInt16BE.call(data, 0));
+          var length = readUInt16BE.call(data, 0);
+          if (self.maxPayloadExceeded(length)){
+            self.error('Max payload exceeded in compressed text message. Aborting...', 1009);
+            return;
+          }
+          opcodes['2'].getData.call(self, length);
         });
       }
       else if (firstLength == 127) {
@@ -503,7 +571,12 @@ var opcodes = {
             self.error('packets with length spanning more than 32 bit is currently not supported', 1008);
             return;
           }
-          opcodes['2'].getData.call(self, readUInt32BE.call(data, 4, true));
+          var length = readUInt32BE.call(data, 4, true);
+          if (self.maxPayloadExceeded(length)){
+            self.error('Max payload exceeded in compressed text message. Aborting...', 1009);
+            return;
+          }
+          opcodes['2'].getData.call(self, length);
         });
       }
     },
@@ -529,11 +602,29 @@ var opcodes = {
       var state = clone(this.state);
       this.messageHandlers.push(function(callback) {
         self.applyExtensions(packet, state.lastFragment, state.compressed, function(err, buffer) {
-          if (err) return self.error(err.message, 1007);
-          if (buffer != null) self.currentMessage.push(buffer);
+          if (err) {
+            if(err.type===1009){
+                return self.error('Max payload exceeded in compressed binary message. Aborting...', 1009);
+            }
+            return self.error(err.message, 1007);
+          }
+          if (buffer != null) {
+            if( self.maxPayload==0 || (self.maxPayload > 0 && (self.currentMessageLength + buffer.length) < self.maxPayload) ){
+              self.currentMessage.push(buffer);
+            }
+            else{
+                self.currentMessage=null;
+                self.currentMessage = [];
+                self.currentMessageLength = 0;
+                self.error(new Error('Maximum payload exceeded'), 1009);
+                return;
+            }
+            self.currentMessageLength += buffer.length;
+          }
           if (state.lastFragment) {
-            var messageBuffer = self.concatBuffers(self.currentMessage);
+            var messageBuffer = Buffer.concat(self.currentMessage);
             self.currentMessage = [];
+            self.currentMessageLength = 0;
             self.onbinary(messageBuffer, {masked: state.masked, buffer: messageBuffer});
           }
           callback();
