@@ -185,6 +185,21 @@ describe('WebSocketServer', function() {
         }
       });
     });
+    it('will not crash when it receives an unhandled opcode', function(done) {
+      var wss = new WebSocketServer({ port: 8080 });
+      wss.on('connection', function connection(ws) {
+          ws.onerror = function(error) {
+              done();
+          };
+      });
+
+      var socket = new WebSocket('ws://127.0.0.1:8080/');
+
+      socket.onopen = function() {
+          socket._socket.write(new Buffer([5]));
+          socket.send('');
+      };
+    });  
   });
 
   describe('#close', function() {
@@ -232,6 +247,18 @@ describe('WebSocketServer', function() {
       });
     });
 
+    it('cleans event handlers on precreated server', function(done) {
+      var srv = http.createServer();
+      srv.listen(++port, function() {
+        var wss = new WebSocketServer({server: srv});
+        wss.close();
+        srv.emit('upgrade');
+        srv.on('error', function() {});
+        srv.emit('error');
+        done()
+      });
+    });
+
     it('cleans up websocket data on a precreated server', function(done) {
       var srv = http.createServer();
       srv.listen(++port, function () {
@@ -315,6 +342,49 @@ describe('WebSocketServer', function() {
     });
   });
 
+  describe('#maxpayload #hybiOnly', function() {
+    it('maxpayload is passed on to clients,', function(done) {
+      var _maxPayload = 20480;
+      var wss = new WebSocketServer({port: ++port,maxPayload:_maxPayload, disableHixie: true}, function() {
+        wss.clients.length.should.eql(0);
+        var ws = new WebSocket('ws://localhost:' + port);
+      });
+      wss.on('connection', function(client) {
+        wss.clients.length.should.eql(1);
+        wss.clients[0].maxPayload.should.eql(_maxPayload);
+        wss.close();
+        done();
+      });
+    });
+    it('maxpayload is passed on to hybi receivers', function(done) {
+      var _maxPayload = 20480;
+      var wss = new WebSocketServer({port: ++port,maxPayload:_maxPayload,  disableHixie: true}, function() {
+        wss.clients.length.should.eql(0);
+        var ws = new WebSocket('ws://localhost:' + port);
+      });
+      wss.on('connection', function(client) {
+        wss.clients.length.should.eql(1);
+        wss.clients[0]._receiver.maxPayload.should.eql(_maxPayload);
+        wss.close();
+        done();
+      });
+    });
+    it('maxpayload is passed on to permessage-deflate', function(done) {
+      var PerMessageDeflate = require('../lib/PerMessageDeflate');
+      var _maxPayload = 20480;
+      var wss = new WebSocketServer({port: ++port,maxPayload:_maxPayload, disableHixie: true}, function() {
+        wss.clients.length.should.eql(0);
+        var ws = new WebSocket('ws://localhost:' + port);
+      });
+      wss.on('connection', function(client) {
+        wss.clients.length.should.eql(1);
+        wss.clients[0]._receiver.extensions[PerMessageDeflate.extensionName]._maxPayload.should.eql(_maxPayload);
+        wss.close();
+        done();
+      });
+    });
+  });
+
   describe('#handleUpgrade', function() {
     it('can be used for a pre-existing server', function (done) {
       var srv = http.createServer();
